require 'tennpipes-assist'
require 'i18n'
require 'enumerator'
require 'active_support/time_with_zone'               # next extension depends on this
require 'active_support/core_ext/string/conversions'  # to_date
require 'active_support/option_merger'                # with_options
require 'active_support/core_ext/object/with_options' # with_options
require 'active_support/inflector'                    # humanize
require 'active_support/core_ext/hash/except'         # Hash#except
require 'tennpipes/rendering'

FileSet.glob_require('tennpipes-helper/**/*.rb', __FILE__)
I18n.load_path += Dir["#{File.dirname(__FILE__)}/tennpipes-helper/locale/*.yml"]
I18n.enforce_available_locales = true

module Tennpipes
  ##
  # This component provides a variety of view helpers related to html markup generation.
  # There are helpers for generating tags, forms, links, images, and more.
  # Most of the basic methods should be very familiar to anyone who has used rails view helpers.
  #
  module Helpers
    class << self
      ##
      # Registers these helpers into your application:
      #
      #   Tennpipes::Helpers::OutputHelpers
      #   Tennpipes::Helpers::TagHelpers
      #   Tennpipes::Helpers::AssetTagHelpers
      #   Tennpipes::Helpers::FormHelpers
      #   Tennpipes::Helpers::FormatHelpers
      #   Tennpipes::Helpers::RenderHelpers
      #   Tennpipes::Helpers::NumberHelpers
      #
      # @param [Sinatra::Application] app
      #   The specified Tennpipes application.
      #
      # @example Register the helper module
      #   require 'tennpipes-helper'
      #   class Tennpipes::Application
      #     register Tennpipes::Helpers
      #   end
      #
      def registered(app)
        app.register Tennpipes::Rendering
        app.set :default_builder, 'StandardFormBuilder'
        included(app)
      end

      def included(base)
        base.send :include, Tennpipes::Helpers::OutputHelpers
        base.send :include, Tennpipes::Helpers::TagHelpers
        base.send :include, Tennpipes::Helpers::AssetTagHelpers
        base.send :include, Tennpipes::Helpers::FormHelpers
        base.send :include, Tennpipes::Helpers::FormatHelpers
        base.send :include, Tennpipes::Helpers::RenderHelpers
        base.send :include, Tennpipes::Helpers::NumberHelpers
        base.send :include, Tennpipes::Helpers::TranslationHelpers
      end
    end
  end
end
